package javax.swing.plaf;

import java.awt.Dimension;
import javax.swing.JPanel;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.HierarchyEvent;
import java.awt.event.InputMethodEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.FocusEvent;
import javax.swing.JLayer;
import java.awt.AWTEvent;
import javax.swing.JComponent;
import java.awt.Graphics;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import java.awt.Component;

public class LayerUI<V extends Component> extends ComponentUI implements Serializable
{
    private final PropertyChangeSupport propertyChangeSupport;
    
    public LayerUI() {
        this.propertyChangeSupport = new PropertyChangeSupport(this);
    }
    
    @Override
    public void paint(Graphics g, JComponent component) {
        component.paint(g);
    }
    
    public void eventDispatched(AWTEvent awtEvent, JLayer<? extends V> layer) {
        if ((awtEvent instanceof FocusEvent)) {
            this.processFocusEvent((FocusEvent)awtEvent, layer);
        }
        else if ((awtEvent instanceof MouseEvent)) {
            switch (awtEvent.getID()) {
                case 500:
                case 501:
                case 502:
                case 504:
                case 505: {
                    this.processMouseEvent((MouseEvent)awtEvent, layer);
                    break;
                }
                case 503:
                case 506: {
                    this.processMouseMotionEvent((MouseEvent)awtEvent, layer);
                    break;
                }
                case 507: {
                    this.processMouseWheelEvent((MouseWheelEvent)awtEvent, layer);
                    break;
                }
            }
        }
        else if ((awtEvent instanceof KeyEvent)) {
            this.processKeyEvent((KeyEvent)awtEvent, layer);
        }
        else if ((awtEvent instanceof ComponentEvent)) {
            this.processComponentEvent((ComponentEvent)awtEvent, layer);
        }
        else if ((awtEvent instanceof InputMethodEvent)) {
            this.processInputMethodEvent((InputMethodEvent)awtEvent, layer);
        }
        else if ((awtEvent instanceof HierarchyEvent)) {
            switch (awtEvent.getID()) {
                case 1400: {
                    this.processHierarchyEvent((HierarchyEvent)awtEvent, layer);
                    break;
                }
                case 1401:
                case 1402: {
                    this.processHierarchyBoundsEvent((HierarchyEvent)awtEvent, layer);
                    break;
                }
            }
        }
    }
    
    protected void processComponentEvent(ComponentEvent componentEvent, JLayer<? extends V> layer) {
    }
    
    protected void processFocusEvent(FocusEvent focusEvent, JLayer<? extends V> layer) {
    }
    
    protected void processKeyEvent(KeyEvent keyEvent, JLayer<? extends V> layer) {
    }
    
    protected void processMouseEvent(MouseEvent mouseEvent, JLayer<? extends V> layer) {
    }
    
    protected void processMouseMotionEvent(MouseEvent mouseEvent, JLayer<? extends V> layer) {
    }
    
    protected void processMouseWheelEvent(MouseWheelEvent mouseWheelEvent, JLayer<? extends V> layer) {
    }
    
    protected void processInputMethodEvent(InputMethodEvent inputMethodEvent, JLayer<? extends V> layer) {
    }
    
    protected void processHierarchyEvent(HierarchyEvent hierarchyEvent, JLayer<? extends V> layer) {
    }
    
    protected void processHierarchyBoundsEvent(HierarchyEvent hierarchyEvent, JLayer<? extends V> layer) {
    }
    
    public void updateUI(JLayer<? extends V> layer) {
    }
    
    @Override
    public void installUI(JComponent component) {
        this.addPropertyChangeListener((JLayer)component);
    }
    
    @Override
    public void uninstallUI(JComponent component) {
        this.removePropertyChangeListener((JLayer)component);
    }
    
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        this.propertyChangeSupport.addPropertyChangeListener(listener);
    }
    
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        this.propertyChangeSupport.removePropertyChangeListener(listener);
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners() {
        return this.propertyChangeSupport.getPropertyChangeListeners();
    }
    
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.propertyChangeSupport.addPropertyChangeListener(propertyName, listener);
    }
    
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        this.propertyChangeSupport.removePropertyChangeListener(propertyName, listener);
    }
    
    public PropertyChangeListener[] getPropertyChangeListeners(String propertyName) {
        return this.propertyChangeSupport.getPropertyChangeListeners(propertyName);
    }
    
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        this.propertyChangeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    public void applyPropertyChange(PropertyChangeEvent propertyChangeEvent, JLayer<? extends V> layer) {
    }
    
    @Override
    public int getBaseline(JComponent component, int width, int height) {
        JLayer<Object> layer = (JLayer)component;
        if (layer.getView() != null) {
            return layer.getView().getBaseline(width, height);
        }
        return super.getBaseline(component, width, height);
    }
    
    @Override
    public Component.BaselineResizeBehavior getBaselineResizeBehavior(JComponent component) {
        JLayer<Object> layer = (JLayer)component;
        if (layer.getView() != null) {
            return layer.getView().getBaselineResizeBehavior();
        }
        return super.getBaselineResizeBehavior(component);
    }
    
    public void doLayout(JLayer<? extends V> layer) {
        Component view = layer.getView();
        if (view != null) {
            view.setBounds(0, 0, layer.getWidth(), layer.getHeight());
        }
        JPanel glassPane = layer.getGlassPane();
        if (glassPane != null) {
            glassPane.setBounds(0, 0, layer.getWidth(), layer.getHeight());
        }
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        Component view = ((JLayer)component).getView();
        if (view != null) {
            return view.getPreferredSize();
        }
        return super.getPreferredSize(component);
    }
    
    @Override
    public Dimension getMinimumSize(JComponent component) {
        Component view = ((JLayer)component).getView();
        if (view != null) {
            return view.getMinimumSize();
        }
        return super.getMinimumSize(component);
    }
    
    @Override
    public Dimension getMaximumSize(JComponent component) {
        Component view = ((JLayer)component).getView();
        if (view != null) {
            return view.getMaximumSize();
        }
        return super.getMaximumSize(component);
    }
    
    public void paintImmediately(int x, int y, int w, int h, JLayer<? extends V> layer) {
        layer.paintImmediately(x, y, w, h);
    }
}
